@@ -1,9 +1,9 @@
 use strict;
 use warnings;
 package Email::Address;
-# ABSTRACT: RFC 2822 Address Parsing and Creation
-$Email::Address::VERSION = '1.904'; # TRIAL
-our $COMMENT_NEST_LEVEL ||= 2;
+# ABSTRACT: (DEPRECATED) RFC 2822 Address Parsing and Creation
+$Email::Address::VERSION = '1.910'; # TRIAL
+our $COMMENT_NEST_LEVEL ||= 1;
 our $STRINGIFY          ||= 'format';
 our $COLLAPSE_SPACES      = 1 unless defined $COLLAPSE_SPACES; # I miss //=
 
@@ -16,12 +16,16 @@ our $COLLAPSE_SPACES      = 1 unless defined $COLLAPSE_SPACES; # I miss //=
 #pod
 #pod   print $address->format;
 #pod
-#pod =head1 VERSION
-#pod
-#pod version 1.898
-#pod
 #pod =head1 DESCRIPTION
 #pod
+#pod B<ACHTUNG!> This module has a vulnerability
+#pod (L<CVE-2015-7686|https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2015-7686>)
+#pod which allows remote attackers to cause denial of service.   In other words,
+#pod sometimes it takes way too long to process certain kinds of input.  Maybe
+#pod someday this will be fixed.  Until then, use
+#pod L<B<Email::Address::XS>|Email::Address::XS> instead which has backward
+#pod compatible API.
+#pod
 #pod This class implements a regex-based RFC 2822 parser that locates email
 #pod addresses in strings and returns a list of C<Email::Address> objects found.
 #pod Alternatively you may construct objects manually. The goal of this software is
@@ -40,18 +44,18 @@ my $ctext          = qr/(?>[^()\\]+)/;
 my ($ccontent, $comment) = (q{})x2;
 for (1 .. $COMMENT_NEST_LEVEL) {
   $ccontent = qr/$ctext|$quoted_pair|$comment/;
-  $comment  = qr/\s*\((?:\s*$ccontent)*\s*\)\s*/;
+  $comment  = qr/(?>\s*\((?:\s*$ccontent)*\s*\)\s*)/;
 }
-my $cfws           = qr/$comment|\s+/;
+my $cfws           = qr/$comment|(?>\s+)/;
 
 my $atext          = qq/[^$CTL$special\\s]/;
-my $atom           = qr/$cfws*$atext+$cfws*/;
-my $dot_atom_text  = qr/$atext+(?:\.$atext+)*/;
-my $dot_atom       = qr/$cfws*$dot_atom_text$cfws*/;
+my $atom           = qr/(?>$cfws*$atext+$cfws*)/;
+my $dot_atom_text  = qr/(?>$atext+(?:\.$atext+)*)/;
+my $dot_atom       = qr/(?>$cfws*$dot_atom_text$cfws*)/;
 
 my $qtext          = qr/[^\\"]/;
 my $qcontent       = qr/$qtext|$quoted_pair/;
-my $quoted_string  = qr/$cfws*"$qcontent+"$cfws*/;
+my $quoted_string  = qr/(?>$cfws*"$qcontent*"$cfws*)/;
 
 my $word           = qr/$atom|$quoted_string/;
 
@@ -67,15 +71,15 @@ my $word           = qr/$atom|$quoted_string/;
 # So we disallow the hateful CFWS in this context for now.  Of modern mail
 # agents, only Apple Web Mail 2.0 is known to produce obs-phrase.
 # -- rjbs, 2006-11-19
-my $simple_word    = qr/$atom|\.|\s*"$qcontent+"\s*/;
-my $obs_phrase     = qr/$simple_word+/;
+my $simple_word    = qr/(?>$atom|\.|\s*"$qcontent+"\s*)/;
+my $obs_phrase     = qr/(?>$simple_word+)/;
 
-my $phrase         = qr/$obs_phrase|(?:$word+)/;
+my $phrase         = qr/$obs_phrase|(?>$word+)/;
 
 my $local_part     = qr/$dot_atom|$quoted_string/;
 my $dtext          = qr/[^\[\]\\]/;
 my $dcontent       = qr/$dtext|$quoted_pair/;
-my $domain_literal = qr/$cfws*\[(?:\s*$dcontent)*\s*\]$cfws*/;
+my $domain_literal = qr/(?>$cfws*\[(?:\s*$dcontent)*\s*\]$cfws*)/;
 my $domain         = qr/$dot_atom|$domain_literal/;
 
 my $display_name   = $phrase;
@@ -128,9 +132,9 @@ my $display_name   = $phrase;
 #pod =cut
 
 our $addr_spec  = qr/$local_part\@$domain/;
-our $angle_addr = qr/$cfws*<$addr_spec>$cfws*/;
+our $angle_addr = qr/(?>$cfws*<$addr_spec>$cfws*)/;
 our $name_addr  = qr/(?>$display_name?)$angle_addr/;
-our $mailbox    = qr/(?:$name_addr|$addr_spec)$comment*/;
+our $mailbox    = qr/(?:$name_addr|$addr_spec)(?>$comment*)/;
 
 sub _PHRASE   () { 0 }
 sub _ADDRESS  () { 1 }
@@ -157,14 +161,20 @@ sub __dump {
 #pod     q[me@local, Casey <me@local>, "Casey" <me@local> (West)]
 #pod   );
 #pod
+#pod B<ACHTUNG!> This is where that vulnerability mentioned above lies.  Do not use
+#pod this method with untrusted user input.
+#pod
+#pod Use method L<parse from the Email::Address::XS module|Email::Address::XS/parse>
+#pod instead.
+#pod
 #pod This method returns a list of C<Email::Address> objects it finds in the input
 #pod string.  B<Please note> that it returns a list, and expects that it may find
 #pod multiple addresses.  The behavior in scalar context is undefined.
 #pod
 #pod The specification for an email address allows for infinitely nestable comments.
 #pod That's nice in theory, but a little over done.  By default this module allows
-#pod for two (C<2>) levels of nested comments. If you think you need more, modify
-#pod the C<$Email::Address::COMMENT_NEST_LEVEL> package variable to allow more.
+#pod for one (C<1>) level of nested comments. If you think you need more, modify the
+#pod C<$Email::Address::COMMENT_NEST_LEVEL> package variable to allow more.
 #pod
 #pod   $Email::Address::COMMENT_NEST_LEVEL = 10; # I'm deep
 #pod
@@ -212,7 +222,13 @@ sub parse {
         return @cached;
     }
 
-    my (@mailboxes) = ($line =~ /$mailbox/go);
+    my %mailboxes;
+    my $str = $line;
+    $str =~ s!($name_addr(?>$comment*))!$mailboxes{pos($str)} = $1; ',' x length $1!ego
+        if $str =~ /$angle_addr/;
+    $str =~ s!($addr_spec(?>$comment*))!$mailboxes{pos($str)} = $1; ',' x length $1!ego;
+    my @mailboxes = map { $mailboxes{$_} } sort { $a <=> $b } keys %mailboxes;
+
     my @addrs;
     foreach (@mailboxes) {
       my $original = $_;
@@ -239,6 +255,8 @@ sub parse {
         $_ = undef unless length $_;
       }
 
+      $phrase =~ s/\\(.)/$1/g if $phrase;
+
       my $new_comment = join q{ }, @comments;
       push @addrs,
         $class->new($phrase, "$user\@$host", $new_comment, $original);
@@ -534,11 +552,11 @@ use overload '""' => 'as_string', fallback => 1;
 
 =head1 NAME
 
-Email::Address - RFC 2822 Address Parsing and Creation
+Email::Address - (DEPRECATED) RFC 2822 Address Parsing and Creation
 
 =head1 VERSION
 
-version 1.904
+version 1.910
 
 =head1 SYNOPSIS
 
@@ -551,6 +569,14 @@ version 1.904
 
 =head1 DESCRIPTION
 
+B<ACHTUNG!> This module has a vulnerability
+(L<CVE-2015-7686|https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2015-7686>)
+which allows remote attackers to cause denial of service.   In other words,
+sometimes it takes way too long to process certain kinds of input.  Maybe
+someday this will be fixed.  Until then, use
+L<B<Email::Address::XS>|Email::Address::XS> instead which has backward
+compatible API.
+
 This class implements a regex-based RFC 2822 parser that locates email
 addresses in strings and returns a list of C<Email::Address> objects found.
 Alternatively you may construct objects manually. The goal of this software is
@@ -611,14 +637,20 @@ following comment.
     q[me@local, Casey <me@local>, "Casey" <me@local> (West)]
   );
 
+B<ACHTUNG!> This is where that vulnerability mentioned above lies.  Do not use
+this method with untrusted user input.
+
+Use method L<parse from the Email::Address::XS module|Email::Address::XS/parse>
+instead.
+
 This method returns a list of C<Email::Address> objects it finds in the input
 string.  B<Please note> that it returns a list, and expects that it may find
 multiple addresses.  The behavior in scalar context is undefined.
 
 The specification for an email address allows for infinitely nestable comments.
 That's nice in theory, but a little over done.  By default this module allows
-for two (C<2>) levels of nested comments. If you think you need more, modify
-the C<$Email::Address::COMMENT_NEST_LEVEL> package variable to allow more.
+for one (C<1>) level of nested comments. If you think you need more, modify the
+C<$Email::Address::COMMENT_NEST_LEVEL> package variable to allow more.
 
   $Email::Address::COMMENT_NEST_LEVEL = 10; # I'm deep
 
@@ -784,10 +816,6 @@ On his 1.8GHz Apple MacBook, rjbs gets these results:
 certain known characteristics, and disabling cache will also degrade
 performance.
 
-=head1 VERSION
-
-version 1.898
-
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Kevin Riggle and Tatsuhiko Miyagawa for tests for annoying
@@ -807,6 +835,50 @@ Ricardo SIGNES <rjbs@cpan.org>
 
 =back
 
+=head1 CONTRIBUTORS
+
+=for stopwords Alex Vandiver David Golden Steinbrunner Glenn Fowler Kevin Falcone Pali Ruslan Zakirov sunnavy William Yardley
+
+=over 4
+
+=item *
+
+Alex Vandiver <alex@chmrr.net>
+
+=item *
+
+David Golden <dagolden@cpan.org>
+
+=item *
+
+David Steinbrunner <dsteinbrunner@pobox.com>
+
+=item *
+
+Glenn Fowler <cebjyre@cpan.org>
+
+=item *
+
+Kevin Falcone <kevin@jibsheet.com>
+
+=item *
+
+Pali <pali@cpan.org>
+
+=item *
+
+Ruslan Zakirov <ruz@bestpractical.com>
+
+=item *
+
+sunnavy <sunnavy@bestpractical.com>
+
+=item *
+
+William Yardley <pep@veggiechinese.net>
+
+=back
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2004 by Casey West.
